'''Tweaks for the SOAP library, for fixing SOAP implementation issues'''


## These are needed for dump_list (or any other SOAPBuilder method)
from SOAPpy.Config import Config
from SOAPpy.NS import NS
from SOAPpy.Types import *
from SOAPpy.wstools.XMLname import toXMLname

try:              basestring
except NameError: basestring = (str, unicode) ## basestring replecement for python < 2.3

## Redefine the function to dump lists, so we can tweak namespace and encoding
def dump_list(self, obj, tag, typed = 1, ns_map = {}):
    if Config.debug: print "In dump_list.", "obj=", obj
    tag = tag or self.gentag()
    tag = toXMLname(tag) # convert from SOAP 1.2 XML name encoding

    if type(obj) == InstanceType:
        data = obj.data
    else:
        data = obj

    id = self.checkref(obj, tag, ns_map)
    if id is None:
        return

    same_type = 1 ## Assume all are the same, unless we discover otherwise
    try:
        sample = data[0]
    except:
        sample = structType()
        ## preserve type if present
        try:
            sample._typename = obj._typed and obj._type or 'SOAPStruct'
        except AttributeError:
            pass
    else:
        for x in data[1:]:
            if sample.__class__ != x.__class__:
                same_type = 0
                break

    ndecl = ''
    if same_type:
        if isinstance(sample, structType) or (sample is dict): # force to urn struct
            try:
                tns = obj._ns or NS.URN
            except:
                tns = NS.URN
            ns, ndecl = self.genns(ns_map, tns)
            try:
                typename = sample._typename
            except:
                typename = 'SOAPStruct'
            t = ns + typename
        elif isinstance(sample, anyType):
            ns = sample._validNamespaceURI(self.config.typesNamespaceURI,
                                           self.config.strictNamespaces)
            if ns:
                ns, ndecl = self.genns(ns_map, ns)
                t = ns + str(sample._type)
            else:
                t = 'ur-type'
        else:
            typename = type(sample).__name__
            ## - basestring covers 'str' and 'unicode' which are 'string' in SOAP
            ## - python's 'long'  is 'integer' in SOAP
            ## - python's 'float' is 'double'  in SOAP
            if isinstance(sample, basestring): typename = 'string'
            if typename == 'long':  typename = 'integer'
            if typename == 'float': typename = 'double'
            t = self.genns(ns_map, self.config.typesNamespaceURI)[0] + typename
    else:
        t = self.genns(ns_map, self.config.typesNamespaceURI)[0] + 'ur-type'

    try: a = obj._marshalAttrs(ns_map, self)
    except: a = ''

    ens, edecl = self.genns(ns_map, NS.ENC)
    ins, idecl = self.genns(ns_map, self.config.schemaNamespaceURI)

    self.out.append(
        '<%s %sarrayType="%s[%d]" %stype="%sArray"%s%s%s%s%s%s>\n' %
        (tag, ens, t, len(data), ins, ens, ndecl, edecl, idecl,
         self.genroot(ns_map), id, a))

    try:
        elemsname = obj._elemsname
    except:
        elemsname = 'item'
        
    for i in data:
        self.dump(i, elemsname, not same_type, ns_map)
        
    self.out.append('</%s>\n' % tag)


def do_GET(self):
    path = self.path.lower()
    if path.endswith('wsdl'):
        method = 'wsdl'
        function = namespace = None
        if (self.server.funcmap.has_key(namespace)
            and self.server.funcmap[namespace].has_key(method)):
            function = self.server.funcmap[namespace][method]
        else: 
            if namespace in self.server.objmap.keys():
                function = self.server.objmap[namespace]
                l = method.split(".")
                for i in l:
                    function = getattr(function, i)

        if function:
            response = function(self.path)
            if response is not None:
                self.send_response(200)
                self.send_header("Content-type", 'text/xml')
                self.end_headers()
                self.wfile.write(str(response))
                return

    # return error
    self.send_response(200)
    self.send_header("Content-type", 'text/html')
    self.end_headers()
    self.wfile.write('''\
<html>
 <head><title>Error!</title></head>
 <body>
  <h1>Oops!</h1>

  <p>
   This server supports HTTP GET requests only for the the purpose of
   obtaining Web Services Description Language (WSDL) for a specific
   service.

   Either you requested an URL that does not point to a WSDL file or
   this server does not implement a wsdl retrieval method.
  </p>
 </body>
</html>
''')



## Now we tweak what we need

import new
from SOAPpy import SOAPBuilder
from SOAPpy.Server import SOAPRequestHandler

method = new.instancemethod(dump_list, None, SOAPBuilder)
setattr(SOAPBuilder, 'dump_list', method)
setattr(SOAPBuilder, 'dump_tuple', method)

method = new.instancemethod(do_GET, None, SOAPRequestHandler)
setattr(SOAPRequestHandler, 'do_GET', method)

